@@ -31,6 +31,8 @@
 
 #include <math.h>
 
+#define LINE_WIDTH 0.5
+
 typedef struct
 {
   GtkWidget      *overflow_popover;
@@ -121,8 +123,6 @@ static gboolean       gcal_month_view_key_press             (GtkWidget      *wid
 static void           add_new_event_button_cb               (GtkWidget      *button,
                                                              gpointer        user_data);
 
-static void           gcal_view_interface_init              (GcalViewIface  *iface);
-
 static void           gcal_month_view_set_property          (GObject        *object,
                                                              guint           property_id,
                                                              const GValue   *value,
@@ -175,6 +175,8 @@ static void           gcal_month_view_clear_marks           (GcalView       *vie
 static GList*         gcal_month_view_get_children_by_uuid  (GcalView       *view,
                                                              const gchar    *uuid);
 
+static void           gcal_view_interface_init              (GcalViewInterface *iface);
+
 G_DEFINE_TYPE_WITH_CODE (GcalMonthView, gcal_month_view,GCAL_TYPE_SUBSCRIBER_VIEW,
                          G_ADD_PRIVATE (GcalMonthView)
                          G_IMPLEMENT_INTERFACE (GCAL_TYPE_VIEW, gcal_view_interface_init));
@@ -276,11 +278,18 @@ show_popover_for_position (GcalMonthView *view,
           start_day = aux;
         }
 
-      start_dt = g_date_time_new_utc (priv->date->year, priv->date->month, start_day, 0, 0, 0);
+      start_dt = g_date_time_new_local (priv->date->year, priv->date->month, start_day, 0, 0, 0);
 
       /* Only setup an end date when days are different */
       if (start_day != end_day)
-        end_dt = g_date_time_new_utc (priv->date->year, priv->date->month, end_day + 1, 0, 0, 0);
+        {
+          GDateTime *tmp_dt;
+
+          tmp_dt = g_date_time_new_local (priv->date->year, priv->date->month, end_day, 0, 0, 0);
+          end_dt = g_date_time_add_days (tmp_dt, 1);
+
+          g_clear_pointer (&tmp_dt, g_date_time_unref);
+        }
 
       g_signal_emit_by_name (GCAL_VIEW (widget), "create-event", start_dt, end_dt, x, y);
 
@@ -513,11 +522,7 @@ rebuild_popover_for_day (GcalMonthView *view,
                                   day,
                                   0, 0, 0);
 
-      dt_end = g_date_time_new (tz,
-                                g_date_time_get_year (current_date),
-                                g_date_time_get_month (current_date),
-                                day + 1,
-                                0, 0, 0);
+      dt_end = g_date_time_add_days (dt_start, 1);
 
       for (; l != NULL; l = g_list_next (l))
         {
@@ -661,7 +666,7 @@ gcal_month_view_key_press (GtkWidget   *widget,
   GcalMonthViewPrivate *priv;
   gboolean selection, valid_key;
   gdouble x, y;
-  gint min, max, diff;
+  gint min, max, diff, month_change;
 
   g_return_val_if_fail (GCAL_IS_MONTH_VIEW (widget), FALSE);
 
@@ -669,6 +674,7 @@ gcal_month_view_key_press (GtkWidget   *widget,
   selection = event->state & GDK_SHIFT_MASK;
   valid_key = FALSE;
   diff = 0;
+  month_change = 0;
   min = priv->days_delay;
   max = priv->days_delay + icaltime_days_in_month (priv->date->month, priv->date->year) - 1;
 
@@ -728,6 +734,28 @@ gcal_month_view_key_press (GtkWidget   *widget,
     {
       priv->keyboard_cell += diff;
     }
+  else
+    {
+      month_change = priv->keyboard_cell + diff > max ? 1 : -1;
+      priv->date->month += month_change;
+      *(priv->date) = icaltime_normalize(*(priv->date));
+
+      priv->days_delay = (time_day_of_week (1, priv->date->month - 1, priv->date->year) - priv->first_weekday + 7) % 7;
+
+      /*
+       * Set keyboard cell value to the sum or difference of days delay of successive
+       * month or last day of preceeding month and overload value depending on
+       * month_change. Overload value is the equal to the deviation of the value
+       * of keboard_cell from the min or max value of the current month depending
+       * on the overload point.
+       */
+      if (month_change == 1)
+        priv->keyboard_cell = priv->days_delay + priv->keyboard_cell + diff - max - 1;
+      else
+        priv->keyboard_cell = priv->days_delay + icaltime_days_in_month (priv->date->month, priv->date->year) - min + priv->keyboard_cell + diff;
+
+      g_object_notify (G_OBJECT (widget), "active-date");
+    }
 
   if (selection)
     {
@@ -757,7 +785,7 @@ add_new_event_button_cb (GtkWidget *button,
   gtk_widget_hide (priv->overflow_popover);
 
   day = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (priv->overflow_popover), "selected-day"));
-  start_date = g_date_time_new_utc (priv->date->year, priv->date->month, day, 0, 0, 0);
+  start_date = g_date_time_new_local (priv->date->year, priv->date->month, day, 0, 0, 0);
 
   g_signal_emit_by_name (GCAL_VIEW (user_data), "create-event-detailed", start_date, NULL);
 
@@ -841,7 +869,7 @@ gcal_month_view_init (GcalMonthView *self)
 }
 
 static void
-gcal_view_interface_init (GcalViewIface *iface)
+gcal_view_interface_init (GcalViewInterface *iface)
 {
   iface->get_initial_date = gcal_month_view_get_initial_date;
   iface->get_final_date = gcal_month_view_get_final_date;
@@ -872,7 +900,7 @@ gcal_month_view_set_property (GObject       *object,
 
         priv->date = g_value_dup_boxed (value);
         priv->days_delay = (time_day_of_week (1, priv->date->month - 1, priv->date->year) - priv->first_weekday + 7) % 7;
-        priv->keyboard_cell = priv->days_delay;
+        priv->keyboard_cell = priv->days_delay + (priv->date->day - 1);
         priv->start_mark_cell = -1;
         priv->end_mark_cell = -1;
 
@@ -1006,7 +1034,7 @@ gcal_month_view_size_allocate (GtkWidget     *widget,
 {
   GcalSubscriberViewPrivate *ppriv;
   GcalMonthViewPrivate *priv;
-  GtkStyleContext *context;
+  GtkStyleContext *context, *child_context;
 
   gint i, j, sw, shown_rows;
 
@@ -1098,6 +1126,7 @@ gcal_month_view_size_allocate (GtkWidget     *widget,
       event = gcal_event_widget_get_event (l->data);
       uuid = gcal_event_get_uid (event);
       all_day = gcal_event_get_all_day (event);
+      child_context = gtk_widget_get_style_context (l->data);
 
       if (!gtk_widget_is_visible (child_widget) && !g_hash_table_contains (ppriv->hidden_as_overflow, uuid))
         continue;
@@ -1194,6 +1223,8 @@ gcal_month_view_size_allocate (GtkWidget     *widget,
                   _gcal_subscriber_view_setup_child (GCAL_SUBSCRIBER_VIEW (widget), child_widget);
                   gtk_widget_show (child_widget);
 
+                  child_context = gtk_widget_get_style_context (child_widget);
+
                   aux = g_hash_table_lookup (ppriv->children, uuid);
                   aux = g_list_append (aux, child_widget);
                 }
@@ -1220,7 +1251,7 @@ gcal_month_view_size_allocate (GtkWidget     *widget,
 
               /* Position and allocate the child widget */
               gtk_style_context_get_margin (gtk_widget_get_style_context (child_widget),
-                                            gtk_style_context_get_state (context),
+                                            gtk_style_context_get_state (child_context),
                                             &margin);
 
               pos_x = cell_width * column + margin.left;
@@ -1293,9 +1324,9 @@ gcal_month_view_size_allocate (GtkWidget     *widget,
 
           if (size_left[i] > natural_height)
             {
-              gtk_style_context_get_margin (gtk_widget_get_style_context (child_widget),
-                                            gtk_style_context_get_state (context),
-                                            &margin);
+              child_context = gtk_widget_get_style_context (child_widget);
+
+              gtk_style_context_get_margin (child_context, gtk_style_context_get_state (child_context), &margin);
 
               pos_x = cell_width * (i % 7) + margin.left;
               pos_y = cell_height * ((i / 7) + first_row_gap) + start_grid_y + margin.top;
@@ -1353,7 +1384,8 @@ gcal_month_view_draw (GtkWidget *widget,
 
   gint font_width, font_height, pos_x, pos_y, shown_rows;
   gint i, j, sw, lower_mark = 43, upper_mark = -2;
-  gdouble start_grid_y, cell_width, cell_height, first_row_gap = 0.0;
+  gint cell_width, cell_height;
+  gdouble start_grid_y, first_row_gap = 0.0;
   gdouble days;
 
   priv = gcal_month_view_get_instance_private (GCAL_MONTH_VIEW (widget));
@@ -1596,25 +1628,25 @@ gcal_month_view_draw (GtkWidget *widget,
   gtk_style_context_add_class (context, "lines");
 
   gtk_style_context_get_color (context, state, &color);
-  cairo_set_line_width (cr, 0.5);
+  cairo_set_line_width (cr, LINE_WIDTH);
   gdk_cairo_set_source_rgba (cr, &color);
   /* vertical lines, the easy ones */
   for (i = 0; i < 6; i++)
     {
       pos_x = cell_width * (i + 1);
-      cairo_move_to (cr, pos_x + 0.4, start_grid_y);
+      cairo_move_to (cr, pos_x + (LINE_WIDTH / 2), start_grid_y);
       cairo_rel_line_to (cr, 0, alloc.height - start_grid_y);
     }
 
   /* top horizontal line */
-  cairo_move_to (cr, 0, start_grid_y + 0.4);
+  cairo_move_to (cr, 0, start_grid_y + (LINE_WIDTH / 2));
   cairo_rel_line_to (cr, alloc.width, 0);
 
   /* drawing weeks lines */
   for (i = 0; i < (shown_rows % 2) + 5; i++)
     {
       pos_y = cell_height * (i + 0.5 * (2.0 - (shown_rows % 2))) + start_grid_y;
-      cairo_move_to (cr, 0, pos_y + 0.4);
+      cairo_move_to (cr, 0, pos_y + (LINE_WIDTH / 2));
       cairo_rel_line_to (cr, alloc.width, 0);
     }
   cairo_stroke (cr);
@@ -1642,11 +1674,11 @@ gcal_month_view_draw (GtkWidget *widget,
      gtk_style_context_restore (context);
 
      gdk_cairo_set_source_rgba (cr, &color);
-     cairo_set_line_width (cr, 0.4);
+     cairo_set_line_width (cr, LINE_WIDTH);
 
-     /* horizontals */
      if ((first_mark / 7) == (last_mark / 7))
        {
+         /* selection in 1 row with 1 rectangular outline */
          gint row = first_mark / 7;
          gint first_column = first_mark % 7;
          gint last_column = last_mark % 7;
@@ -1655,10 +1687,12 @@ gcal_month_view_draw (GtkWidget *widget,
          pos_x2 = cell_width * (last_column + 1);
          pos_y = cell_height * (row + first_row_gap) + start_grid_y;
          pos_y2 = cell_height * (row + 1.0 + first_row_gap) + start_grid_y;
-         cairo_rectangle (cr, pos_x + 0.3, pos_y + 0.3, pos_x2 - pos_x + 0.6, pos_y2 - pos_y + 0.6);
+         cairo_rectangle (cr, pos_x + (LINE_WIDTH / 2), pos_y + (LINE_WIDTH / 2),
+                          pos_x2 - pos_x + (LINE_WIDTH / 2), pos_y2 - pos_y + (LINE_WIDTH / 2));
        }
-     else
+     else if ((first_mark / 7) == ((last_mark / 7) - 1) && (first_mark % 7) > ((last_mark % 7) + 1))
        {
+         /* selection in 2 rows with 2 seperate rectangular outlines */
          gint first_row = first_mark / 7;
          gint last_row = last_mark / 7;
          gint first_column = first_mark % 7;
@@ -1685,12 +1719,57 @@ gcal_month_view_draw (GtkWidget *widget,
 
              pos_y = cell_height * (i + first_row_gap) + start_grid_y;
              pos_y2 = cell_height * (i + 1.0 + first_row_gap) + start_grid_y;
-             cairo_rectangle (cr, pos_x + 0.3, pos_y + 0.3, ((gint)end) + 0.6, pos_y2 - pos_y + 0.6);
+             cairo_rectangle (cr, pos_x + (LINE_WIDTH / 2), pos_y + (LINE_WIDTH / 2),
+                              (gint)end + (LINE_WIDTH / 2), pos_y2 - pos_y + (LINE_WIDTH / 2));
            }
        }
+     else
+       {
+         /* multi-row selection with a single outline */
+         gint first_row = first_mark / 7;
+         gint last_row = last_mark / 7;
+         gint first_column = first_mark % 7;
+         gint last_column = last_mark % 7;
+         gdouble start_x = (cell_width * first_column) + (LINE_WIDTH / 2);
+         gdouble start_y = (cell_height * (first_row + first_row_gap)) + start_grid_y + (LINE_WIDTH / 2);
+         gdouble end_x = (cell_width * (last_column + 1.0)) + (LINE_WIDTH / 2);
+         gdouble end_y = (cell_height * (last_row + first_row_gap + 1.0)) + start_grid_y + (LINE_WIDTH / 2);
+         gdouble max_x = alloc.width - (LINE_WIDTH / 2);
 
-     cairo_stroke (cr);
-   }
+         /* Draw the irregular shaped selection box starting from the
+          * top-left corner of the start date (start_{x,y}) clock-wise
+          * to the right-bottom corner of the end date (end_{x,y}) and
+          * on finishing at the start date again.
+          */
+         cairo_move_to (cr, start_x, start_y);
+         cairo_line_to (cr, max_x, start_y);
+
+         /* Skip intermediate drawing steps if end_{x,y} are at the
+          * last column in the row, this makes sure we always draw the
+          * outline within the visible drawing area.
+          */
+         if (last_column == 6)
+           {
+             cairo_line_to (cr, max_x, end_y);
+           }
+         else
+           {
+             cairo_line_to (cr, max_x, end_y - cell_height);
+             cairo_line_to (cr, end_x, end_y - cell_height);
+             cairo_line_to (cr, end_x, end_y);
+           }
+
+         cairo_line_to (cr, (LINE_WIDTH / 2), end_y);
+         cairo_line_to (cr, (LINE_WIDTH / 2), start_y + cell_height);
+         cairo_line_to (cr, start_x, start_y + cell_height);
+         cairo_line_to (cr, start_x, start_y);
+       }
+
+     cairo_stroke_preserve (cr);
+     /* selection background fill */
+     cairo_set_source_rgba(cr, color.red, color.green, color.blue, 0.1);
+     cairo_fill(cr);
+  }
 
   /* Draw focus rectangle */
   if (priv->keyboard_cell > priv->days_delay - 1)
@@ -1735,7 +1814,7 @@ gcal_month_view_button_press (GtkWidget      *widget,
 
   clicked_cell = real_cell (clicked_cell, priv->k);
 
-  if (clicked_cell >= priv->days_delay && clicked_cell <= days)
+  if (clicked_cell >= priv->days_delay && clicked_cell < days)
     {
       priv->start_mark_cell = clicked_cell;
       priv->keyboard_cell = clicked_cell;
